/**
 * vim: set ts=4 :
 * =============================================================================
 * SourceMod Sample Extension
 * Copyright (C) 2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"

#include "FunctionType.h"

/**
 * @file extension.cpp
 * @brief Implement extension code here.
 */

#define GAMEDATA_FILE "l4d2_CreateSurvivorBot"

CCreateNextBotSurvivorBot g_TheApp;		/**< Global singleton for extension's main interface */
SMEXT_LINK(&g_TheApp);

IServerGameEnts* g_pGameEntitys = NULL;
IGameConfig* g_pGameConfig = NULL;

#include "Natives.h"

bool CCreateNextBotSurvivorBot::SDK_OnLoad(char *error, size_t maxlen, bool late)
{
	//only exec once when server started
	if (!gameconfs->LoadGameConfigFile(GAMEDATA_FILE, &g_pGameConfig, error, maxlen))
	{
		snprintf(error, maxlen, "can't found %s.txt file", GAMEDATA_FILE);
		return false;
	}

	g_pGameConfig->GetMemSig("HandleCommand_JoinTeam", (void**)&fHandleCommand_JoinTeam);
	if (!fHandleCommand_JoinTeam) g_pSM->LogError(myself, "Can't found < HandleCommand_JoinTeam > signature");
	//g_pSM->LogMessage(myself, "fHandleCommand_JoinTeam : %#x", fHandleCommand_JoinTeam);

	g_pGameConfig->GetMemSig("NextBotCreatePlayerBot<SurvivorBot>", (void**)&fNextBotCreatePlayerBotSurvivorBot);
	if (!fNextBotCreatePlayerBotSurvivorBot) g_pSM->LogError(myself, "Can't found < NextBotCreatePlayerBot<SurvivorBot> > signature");

#if _WIN32  // (addr+5) + *(addr+1) = call function addr -> this  signature into <CDirector::AddSurvivorBot>
	if (fNextBotCreatePlayerBotSurvivorBot)
	{
		cell_t offset = (cell_t)fNextBotCreatePlayerBotSurvivorBot + 1;
		offset = *(cell_t*)offset;
		if (offset)
		{
			fNextBotCreatePlayerBotSurvivorBot = (tagNextBotCreatePlayerBotSurvivorBot)(((cell_t)fNextBotCreatePlayerBotSurvivorBot + 5) + offset);
			//g_pSM->LogMessage(myself, "windows: fNextBotCreatePlayerBotSurvivorBot<%#x>", fNextBotCreatePlayerBotSurvivorBot);
		}
		else
		{
			g_pSM->LogError(myself, "windows: Can't found < NextBotCreatePlayerBot<SurvivorBot> > signature");
			fNextBotCreatePlayerBotSurvivorBot = NULL;
		}
	}
#endif
	//g_pSM->LogMessage(myself, "fNextBotCreatePlayerBotSurvivorBot : %#x", fNextBotCreatePlayerBotSurvivorBot);

	g_pGameConfig->GetMemSig("CTerrorPlayer::RoundRespawn", (void**)&fCTerrorPlayerRoundRespawn);
	if (!fCTerrorPlayerRoundRespawn) g_pSM->LogError(myself, "Can't found < fCTerrorPlayerRoundRespawn > signature");

	sharesys->AddNatives(myself, g_theAppNatives);
	return true;
}


void CCreateNextBotSurvivorBot::SDK_OnUnload()
{
	//only exec once when server started
	if(g_pGameConfig) gameconfs->CloseGameConfigFile(g_pGameConfig);
}

bool CCreateNextBotSurvivorBot::SDK_OnMetamodLoad(ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	//only exec once when server started
	//warning: can't using (g_pSM; g_pCVar; ...) in here, because the SourceMod not ready.

	if (ismm->GetSourceEngineBuild() != SOURCE_ENGINE_LEFT4DEAD2)
	{
		snprintf(error, maxlen, "sorry, this extension only suport left 4 dead 2");
		return false;
	}

	GET_V_IFACE_CURRENT(GetEngineFactory, g_pCVar, ICvar, CVAR_INTERFACE_VERSION);//Get CVar pointer. The instance of CVar in engine.dll
	GET_V_IFACE_ANY(GetServerFactory, g_pGameEntitys, IServerGameEnts, INTERFACEVERSION_SERVERGAMEENTS);//Get GameEntity pointer. The instance of GameEntity in server.dll
	return true;
}